# Laravel Google Pub/Sub Queue Driver by Myriad

## Install

### Add Myriad Repository

The client is distributed internally with Satis. First add the repository to `composer.json`:

```json
  "repositories": [
    {
      "type": "composer",
      "url": "https://satis.bushelops.com"
    }
  ]
```

### Require the Dependency

Require this package with composer using the following command:

```bash
composer require myriad/laravel-google-pubsub-queue
```

### Install Service Provider

After updating composer, add the service provider to the `providers` array in `config/app.php`:

```php
Myriad\Queue\Drivers\Google\PubSubQueueServiceProvider::class,
```

## Configuration

### Set up Google Cloud

* Log in to Google Cloud Console and create a Service Account for the laravel application and "Furnish a new private key" in JSON format.
* Add a Topic to Google Pub/Sub; grant the Service Account Pub/Sub Admin privileges on it.
* Click on the Topic and add a Subscription; grant the Service Account Pub/Sub Admin privileges on it.

### Set up Laravel

Add any number of connections using the `google` driver to your queue.php config file:

```php
'google' => [
  'driver' => 'google',
  'queue' => 'topic:subscription',
  'retry_after' => 120,
  'keyFile' => json_decode(base64_decode(env('GOOGLE_KEY_FILE')), true),
],
```

The `queue` property is defined in the format "topic:subscription".
Providing a non-delimited string like "mytopic" is equivalent to "mytopic:mytopic".

The `retry_after` property defines the default timeout **in Pub/Sub** for your jobs. More on timeouts below.

Google Pub/Sub client configuration is merged in, see here for available options:
https://googlecloudplatform.github.io/google-cloud-php/#/docs/google-cloud/v0.32.1/pubsub/pubsubclient?method=__construct

## Usage

The queue driver supports all of Laravel's queue features with the exception of $queue->size() as Pub/Sub 
does not provide a method to get the number of messages on a topic.

__Disclaimer__: Google Pub/Sub does not support delayed messages, however through some clever/dirty usage 
of ack-ing and re-publishing this driver does. As such, every 10 minutes each delayed message must be 
re-acked and every 5 days must be re-published. So although you can delay a message for as long as 
you want, try to keep the number of messages delayed for longer than 10 minutes to a minimum.

### Timeouts

Beginning in Laravel 5.4 with PHP 7.1, the Laravel queue worker internally uses SIGALRM signals to ensure that jobs 
don't live past their `timeout`. This is important, as there is no way for a job to cheat for more time as they could in
the past. Pub/Sub's max timeout is 10 minutes, which means you could run in to a an issue with long running jobs. To 
solve the issue, set the Laravel timeout as high as you would like; Pub/Sub will be informed that the time out is 10 
minutes. At some point before your 10 minutes is up, issue `$this->job->renew($seconds)` in your job to ask Google for 
more time. Once again, you can only ask for 10 minutes, so you will need to do this as many times as needed to keep your
job from being rescheduled on a different worker.
